Imports System.IO
Imports System.Windows.Forms
Imports System.Web.Script.Serialization
Imports System.Drawing.Imaging
Imports System.Text

Public Class Form1
    Public Class projectSettings
        Public Width As Integer
        Public Height As Integer
        Public fontName As String
        Public fontSize As Single
        Public fontStyle As FontStyle
    End Class



    Dim project As New projectSettings

    Public selectedWidget As Widget
    Public selectedWindow As Widget

    ' STARTUP
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        currentDirectoryName = My.Application.Info.DirectoryPath
        applicationDirectory = currentDirectoryName

        loadPreferences() ' LAST FOLDER

        loadSettings() ' EXPORTSETTINGS

        Try
            noImageImage = Bitmap.FromFile(applicationDirectory & "\defaultimage.png")
            noImageBitmap = New Bitmap(noImageImage)
        Catch ex As Exception
            MsgBox("Form1_Load: Unable to load required file - " & ex.Message & " - exiting")
            End
        End Try

        Try
            atlasImage = Bitmap.FromFile(applicationDirectory & "\defaultatlas.png")
            atlasBitmap = New Bitmap(atlasImage)
        Catch ex As Exception
            MsgBox("Form1_Load: Unable to load default atlas - " & ex.Message)
        End Try

        Try
            backdropImage = Bitmap.FromFile(applicationDirectory & "\defaultbackdrop.png")
            backdropBitmap = New Bitmap(backdropImage)
            '            Panel1
        Catch ex As Exception
            MsgBox("Form1_Load: Unable to load default backdrop - " & ex.Message)
        End Try

        Dim guiFont As New Font("OCR A Std", 14, FontStyle.Regular)
        fontLabel.Font = guiFont
        fontLabel.Text = guiFont.Name
        FontDialog1.Font = guiFont
        Panel1.Font = guiFont

        selectedWindow = Nothing

        project.fontName = guiFont.Name
        project.fontSize = guiFont.Size
        project.fontStyle = guiFont.Style

        project.Width = 960
        project.Height = 640
        Panel1.Width = project.Width
        Panel1.Height = project.Height

        singleLineMode = False ' DEFAULT EXPORT OPTIONS
        guiCall = "AGUI."
        eol = vbCrLf
        padOutput = True
        padWidth = 20
        tabWidth = 2

        snapToGrid = gridSnapCheckBox.Checked
        gridSizeX = CInt(snapSizeX.Value)
        gridSizeY = CInt(snapSizeY.Value)

    End Sub
    Private Sub savePreferences()
        Try
            Dim path As String = applicationDirectory & "\Antix GUI Editor.cfg"
            Dim sw As StreamWriter
            Dim newpref As String = ""
            If File.Exists(path) Then
                File.Delete(path) ' delete it before recreating it
            End If
            sw = File.CreateText(path)

            newpref = "LASTDIRECTORY=" & currentDirectoryName
            sw.WriteLine(newpref)

            ' add more here

            sw.Flush()
            sw.Close()
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub
    Private Sub loadPreferences()
        Try
            Dim path As String = applicationDirectory & "\Antix GUI Editor.cfg"
            Dim input As String
            Dim strarray() As String
            Dim sr As StreamReader = File.OpenText(path)
            Do While sr.Peek() <> -1 ' loop through (and process) all lines in prefs file
                input = sr.ReadLine()
                strarray = input.Split("="c)
                Select Case strarray(0) ' key
                    Case "LASTDIRECTORY"
                        currentDirectoryName = strarray(1) ' value
                End Select
            Loop
            sr.Close()
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    ' ATTEMPT RESOURCE CLEANUP
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        savePreferences()

        destroyAllWidgets()
        If atlasImage IsNot Nothing Then
            atlasImage.Dispose()
        End If

        If backdropImage IsNot Nothing Then
            backdropImage.Dispose()
        End If
    End Sub

    ' INTERCEPT KEYPRESSES FOR FORM1
    Private Sub Form1_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        If e.Shift Then
            shiftHeld = True
        End If
        If e.Alt Then
            altHeld = True
        End If

        If e.Control Then
            ctrlHeld = True
        End If
    End Sub
    Private Sub Form1_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyUp
        If Not e.Shift Then
            shiftHeld = False
        End If

        If Not e.Alt Then
            altHeld = False
        End If

        If Not e.Control Then
            ctrlHeld = False
        End If
    End Sub

    ' UTILITY FUNCTIONS
    Private Function clamp(ByVal value As Integer, ByVal min As Integer, ByVal max As Integer) As Integer
        If value < min Then
            Return min
        ElseIf value > max Then
            Return max
        End If
        Return value
    End Function
    Private Function ExtractPathName(ByVal PathFileName As String) As String
        Dim delimiterPosition As Integer = PathFileName.LastIndexOf("\")
        If delimiterPosition > 0 Then
            Return PathFileName.Substring(0, delimiterPosition)
        Else
            Return ""
        End If
    End Function

    ' SHOW ABOUT / HELP WINDOW
    Private Sub aboutButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button16.Click
        Dialog4.ShowDialog()
    End Sub

    ' MANAGE GRID SNAPPING
    Private Sub gridSnap_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles gridSnapCheckBox.CheckedChanged
        snapToGrid = gridSnapCheckBox.Checked
    End Sub
    Private Sub setGridSizeX_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles snapSizeX.ValueChanged
        gridSizeX = CInt(snapSizeX.Value)
    End Sub
    Private Sub snapSizeY_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles snapSizeY.ValueChanged
        gridSizeY = CInt(snapSizeY.Value)
    End Sub

    ' DISPOSE OF ALL WIDGETS
    Private Sub destroyAllWidgets()
        GroupBox2.Enabled = False
        widgetPropertiesGroup.Enabled = False
        remWindow.Enabled = False
        windowListBox.Enabled = False
        Panel1.Controls.Clear()
        For Each w As WidgetItem In windowListBox.Items
            For Each c As Control In w.mWidget.Controls
                Dim widget As Widget = CType(c, Widget)
                destroyWidget(widget) ' DESTROY WIDGET AND ASSOCIATED RESOURCES
            Next
            w.mWidget.Dispose()
        Next
        widgetListBox.Items.Clear()
        windowListBox.Items.Clear()
        GC.Collect()
        selectedWindow = Nothing
    End Sub
    ' START A NEW PROJECT
    Private Sub newButton_Click_1(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles newButton.Click
        Dialog2.displayWidth.Value = 960
        Dialog2.displayHeight.Value = 640
        Dialog2.ShowDialog()
        If Dialog2.DialogResult = System.Windows.Forms.DialogResult.OK Then
            destroyAllWidgets() ' DESTROY OLD PROJECT

            project.Width = CInt(Dialog2.displayWidth.Value)
            project.Height = CInt(Dialog2.displayHeight.Value)
            Panel1.Width = project.Width
            Panel1.Height = project.Height

            'GroupBox3.Enabled = True
            GroupBox1.Enabled = True
            GroupBox5.Enabled = True
            saveButton.Enabled = False
            exportLUA.Enabled = False
            exportJSON.Enabled = False

            UID = 0
        End If
    End Sub

    ' LOAD ATLAS
    Private Sub atlasButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles atlasButton.Click
        OpenFileDialog1.Title = "Select Texture Atlas to Load"
        OpenFileDialog1.FileName = ""
        OpenFileDialog1.InitialDirectory = currentDirectoryName
        OpenFileDialog1.Filter = "PNG Files|*.png"
        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If atlasImage IsNot Nothing Then
                atlasImage.Dispose() ' dispose of the current image if present
                atlasBitmap.Dispose()
            End If
            atlasImage = Bitmap.FromFile(OpenFileDialog1.FileName) ' load the new image
            atlasBitmap = New Bitmap(atlasImage)

            currentDirectoryName = ExtractPathName(OpenFileDialog1.FileName)
        End If
    End Sub
    ' LOAD BACKDROP IMAGE
    Private Sub backdropButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles backdropButton.Click
        OpenFileDialog1.Title = "Select BackDrop Image"
        OpenFileDialog1.FileName = ""
        OpenFileDialog1.InitialDirectory = currentDirectoryName
        OpenFileDialog1.Filter = "PNG Files|*.png"
        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If backdropImage IsNot Nothing Then
                backdropImage.Dispose()
                backdropBitmap.Dispose()
            End If
            backdropImage = Bitmap.FromFile(OpenFileDialog1.FileName)
            backdropBitmap = New Bitmap(backdropImage)
            currentDirectoryName = ExtractPathName(OpenFileDialog1.FileName)
            Panel1.Invalidate()
        End If
    End Sub
    ' TOGGLE BACKDROP SHOWN / HIDDEN
    Private Sub showhideBackdrop_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBox1.CheckedChanged
        If Not CheckBox1.Checked Then
            Panel1.BackColor = Color.DarkGray
        End If
        Panel1.Invalidate()
    End Sub    ' DRAW BACKDROP IMAGE INTO PANEL
    Private Sub Panel1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Panel1.Paint
        If CheckBox1.Checked Then
            If backdropImage IsNot Nothing Then
                Dim sRect As New Rectangle(0, 0, Panel2.Width, Panel2.Height)
                Dim dRect As New Rectangle(0, 0, Panel2.Width, Panel2.Height)
                e.Graphics.DrawImage(backdropImage, dRect, sRect, GraphicsUnit.Pixel)
            End If

        End If
    End Sub

    ' EDITOR WIDGET ENABLING / DISABLING
    Private Sub enableEditingButtons(ByVal widget As Widget)
        Select Case widget.mType
            Case widgetType.WINDOW
                enableWindowButtons()
            Case widgetType.PANEL
                enableBorderButtons()
            Case widgetType.BUTTON
                enableButtonButtons()
            Case widgetType.CHECKBOX
                enableCheckBoxButtons()
            Case widgetType.RADIOBUTTON
                enableRadioButtonButtons()
            Case widgetType.LABEL
                enableLabelButtons()
            Case widgetType.LIST
                enableListButtons()
            Case widgetType.LOG
                enableLogButtons()
            Case widgetType.TEXTVIEW
                enableTextButtons()
            Case widgetType.TEXTINPUT
                enableTextInputButtons()
            Case widgetType.HSCROLLBAR
                enableHScrollBarButtons()
            Case widgetType.VSCROLLBAR
                enableVScrollBarButtons()
            Case widgetType.IMAGE
                enableImageButtons()
        End Select
    End Sub

    Private Sub enableAllButtons(ByVal state As Boolean)
        'widgetName.Enabled = state
        'widgetLeft.Enabled = state
        'widgetTop.Enabled = state
        widgetWidth.Enabled = state
        widgetHeight.Enabled = state
        'widgetCaption.Enabled = state
        widgetHasControls.Enabled = state
        widgetOnValueChanged.Enabled = state
        widgetOnStateChanged.Enabled = state

        widgetZIndex.Enabled = True
        widgetZIndexLabel.Enabled = True

        'widgetTextAlign.Enabled = state
        'widgetTextColor.Enabled = state
        'widgetTextColor.Enabled = state
        'widgetColor.Enabled = state
        'widgetColor.Enabled = state
        'widgetEnabled.Enabled = state
        widgetBorderType.Enabled = state
        'widgetEnabled.Enabled = state
        'widgetHidden.Enabled = state
        widgetOnPress.Enabled = state
        widgetOnRelease.Enabled = state
        widgetOnDrag.Enabled = state

        widgetOrientation.Enabled = state
        widgetMin.Enabled = state
        widgetMax.Enabled = state
        widgetSteps.Enabled = state
        widgetValue.Enabled = state

        widgetInputType.Enabled = state
        widgetOnTextChange.Enabled = state

        widgetRows.Enabled = state

        widgetVisibleLines.Enabled = state
        widgetList.Enabled = state
        widgetIndexChanged.Enabled = state
        widgetItemRemoved.Enabled = state

        widgetTogglestate.Enabled = state
        widgetToggleGroup.Enabled = state

        widgetOnOpen.Enabled = state
        widgetOnClose.Enabled = state

        widgetImageSelect.Enabled = state
        widgetScaleX.Enabled = state
        widgetScaleY.Enabled = state
        widgetRotation.Enabled = state
        widgetRotationLabel.Enabled = state


    End Sub
    Private Sub enableButtonButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetOnPress.Enabled = True
        widgetOnRelease.Enabled = True
        widgetOnDrag.Enabled = True
    End Sub
    Private Sub enableLabelButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetBorderType.Enabled = True
    End Sub
    Private Sub enableTextButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetHeight.Enabled = True
        widgetBorderType.Enabled = True
        widgetVisibleLines.Enabled = True
    End Sub
    Private Sub enableBorderButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetHeight.Enabled = True
        widgetBorderType.Enabled = True
    End Sub
    Private Sub enableHScrollBarButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetMin.Enabled = True
        widgetMax.Enabled = True
        widgetSteps.Enabled = True
        widgetValue.Enabled = True
        widgetOnValueChanged.Enabled = True
    End Sub
    Private Sub enableVScrollBarButtons()
        enableAllButtons(False)
        widgetHeight.Enabled = True
        widgetMin.Enabled = True
        widgetMax.Enabled = True
        widgetSteps.Enabled = True
        widgetValue.Enabled = True
        widgetOnValueChanged.Enabled = True
    End Sub
    Private Sub enableTextInputButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetInputType.Enabled = True
        widgetOnTextChange.Enabled = True
    End Sub
    Private Sub enableLogButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetHeight.Enabled = True
        widgetRows.Enabled = True
    End Sub
    Private Sub enableListButtons()
        enableAllButtons(False)
        widgetWidth.Enabled = True
        widgetHeight.Enabled = True
        widgetVisibleLines.Enabled = True
        widgetList.Enabled = True
        widgetIndexChanged.Enabled = True
        widgetItemRemoved.Enabled = True
    End Sub
    Private Sub enableCheckBoxButtons()
        enableAllButtons(False)
        widgetTogglestate.Enabled = True
        widgetOnStateChanged.Enabled = True
    End Sub
    Private Sub enableRadioButtonButtons()
        enableAllButtons(False)
        widgetTogglestate.Enabled = True
        widgetToggleGroup.Enabled = True
        widgetOnStateChanged.Enabled = True
    End Sub
    Private Sub enableWindowButtons()
        enableAllButtons(False)
        widgetHasControls.Enabled = True
        widgetBorderType.Enabled = True
        widgetOnOpen.Enabled = True
        widgetOnClose.Enabled = True
        widgetWidth.Enabled = True
        widgetHeight.Enabled = True

        widgetZIndex.Enabled = False ' WINDOW ZINDEX IS LOCKED AT 0 (BACKMOST)
        widgetZIndexLabel.Enabled = False
    End Sub
    Private Sub enableImageButtons()
        enableAllButtons(False)
        widgetOnPress.Enabled = True
        widgetOnRelease.Enabled = True
        widgetOnDrag.Enabled = True
        widgetScaleX.Enabled = True
        widgetScaleY.Enabled = True
        widgetImageSelect.Enabled = True
        widgetRotation.Enabled = True
        widgetRotationLabel.Enabled = True
    End Sub






    ' WIDGET SELECTION CODE
    Public Sub getWidgetProperties(ByVal widget As Widget)
        Dim a As Single

        widgetType_ComboBox.SelectedIndex = widget.mType
        widgetName.Text = widget.mName
        widgetLeft.Value = widget.Left
        widgetTop.Value = widget.Top
        widgetWidth.Value = widget.Width
        widgetHeight.Value = widget.Height
        widgetCaption.Text = widget.mCaption
        widgetHasControls.SelectedIndex = widget.mHasControls
        widgetTextAlign.SelectedIndex = widget.mTextAlign

        widgetTextColor.Text = widget.mTextColor
        Panel3.BackColor = Color.FromArgb(widget.mTextColorARGB(1), widget.mTextColorARGB(2), widget.mTextColorARGB(3))

        widgetColor.Text = widget.mColor
        Panel4.BackColor = Color.FromArgb(widget.mColorARGB(1), widget.mColorARGB(2), widget.mColorARGB(3))

        widgetAlpha.Value = widget.mColorARGB(0)
        a = CSng(1 / widgetAlpha.Maximum * widgetAlpha.Value)
        widgetAlphaLabel.Text = a.ToString("F2")

        widgetTextAlpha.Value = widget.mTextColorARGB(0)
        a = CSng(1 / widgetTextAlpha.Maximum * widgetTextAlpha.Value)
        widgetTextAlphaLabel.Text = a.ToString("F2")

        widget.BackColor = Color.FromArgb(widget.mColorARGB(1), widget.mColorARGB(2), widget.mColorARGB(3))

        widgetBorderType.SelectedIndex = widget.mBorderType

        widgetEnabled.SelectedIndex = widget.mEnabled
        widgetHidden.SelectedIndex = widget.mHidden

        widgetTogglestate.SelectedIndex = widget.mToggleState
        widgetToggleGroup.Text = widget.mToggleGroup
        widgetOnStateChanged.Text = widget.mOnStateChanged

        widgetMin.Text = widget.mMin
        widgetMax.Text = widget.mMax
        widgetSteps.Text = widget.mSteps
        widgetValue.Text = widget.mValue
        widgetOnValueChanged.Text = widget.mOnValueChanged
        widgetOrientation.SelectedIndex = widget.mOrientation

        widgetInputType.SelectedIndex = widget.mInputType
        widgetOnTextChange.Text = widget.mOnTextChanged

        widgetVisibleLines.Value = widget.mVisibleLines
        widgetList.Text = widget.mList
        widgetIndexChanged.Text = widget.mOnIndexChanged
        widgetItemRemoved.Text = widget.mOnItemRemoved

        widgetRows.Value = widget.mRows

        widgetOnPress.Text = widget.mOnPress
        widgetOnRelease.Text = widget.mOnRelease
        widgetOnDrag.Text = widget.mOnDrag

        widgetOnClose.Text = widget.mOnClose

        widgetZIndex.Value = CInt(widget.mZIndex * 100)
        widgetZIndexLabel.Text = (widget.mZIndex).ToString("F2")

        widgetScaleX.Text = widget.mScaleX
        widgetScaleY.Text = widget.mScaleY
        widgetImage.Text = widget.mImageName
        widgetRotation.Value = CInt(widget.mImageRotation)
        widgetRotationLabel.Text = widgetRotation.Value.ToString("F2")

    End Sub







    Public Sub selectWidget(ByVal widget As Widget)
        If widget.mType = widgetType.WINDOW Then
            ' HIDE CURRENT WINDOW IF DIFFERENT THAN SELECTED WINDOW
            If selectedWindow IsNot Nothing Then
                If Not ReferenceEquals(selectedWindow, widget) Then
                    selectedWindow.Enabled = False
                    selectedWindow.Visible = False
                    ' REBUILD WIDGET LIST FROM THIS WINDOWS WIDGETS
                    widgetListBox.Items.Clear()
                    If widget.Controls.Count > 0 Then
                        For Each w As Widget In widget.Controls
                            widgetListBox.Items.Add(New WidgetItem(w))
                        Next
                    End If
                End If
            End If
            selectedWidget = widget
            selectedWindow = widget
            windowListBox.SelectedIndex = findWidgetIndex(widget, windowListBox)
            getWidgetProperties(selectedWidget)
            widget.Visible = True
            widget.Enabled = True

            widgetOnClose.Enabled = True
            GroupBox2.Enabled = True
            widgetPropertiesGroup.Enabled = True
        Else
            ' OTHER WIDGET TYPE SELECTED
            selectedWidget = widget
            widgetListBox.SelectedIndex = findWidgetIndex(selectedWidget, widgetListBox)
            getWidgetProperties(selectedWidget)
            'selectedWidget.BringToFront()

            Select Case widget.mType
                Case widgetType.PANEL
                    widget.SendToBack()
                Case Else
                    widget.BringToFront()

            End Select

            widgetOnClose.Enabled = False
        End If
        enableEditingButtons(widget)
    End Sub
    Private Sub widgetListBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetListBox.SelectedIndexChanged
        Dim index As Integer = widgetListBox.SelectedIndex
        If index <> -1 Then
            selectWidget(CType(widgetListBox.Items(index), WidgetItem).mWidget)
        End If
    End Sub
    Private Sub windowListBox_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles windowListBox.SelectedIndexChanged
        Dim index As Integer = windowListBox.SelectedIndex
        If index <> -1 Then
            selectWidget(CType(windowListBox.Items(index), WidgetItem).mWidget)
            selectedWindow.Invalidate() ' FORCE REDRAW
        End If
    End Sub

    ' THIS ACTUALLY FREES ANY ALLOCATED RESOURCES AND THEN DISPOSES OF THE WIDGET IN QESTION
    Private Sub destroyWidget(ByVal w As Widget)
        Select Case w.mType
            Case widgetType.IMAGE
                If w.mImage IsNot Nothing Then ' GET RID OF IMAGERY
                    w.mBitmap.Dispose()
                    w.mImage.Dispose()
                End If
        End Select

        w.Dispose() ' ACTUALLY DESTROT THE WIDGET
    End Sub

    ' DESTROY CURRENTLY SELECTED WIDGET
    Private Sub remWidget_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles remWidget.Click
        If widgetListBox.SelectedIndex <> -1 Then
            If selectedWidget.mType <> widgetType.WINDOW Then
                Dim index As Integer = findWidgetIndex(selectedWidget, widgetListBox)
                destroyWidget(selectedWidget) ' DESTROY WIDGET AND ASSOCIATED RESOURCES
                widgetListBox.Items.RemoveAt(index)
                GC.Collect()
                If widgetListBox.Items.Count = 0 Then
                    remWidget.Enabled = False
                End If
            End If
        End If
    End Sub
    ' DESTROY CURRENTLY SELECTED WINDOW AND ITS CHILD WIDGETS
    Private Sub removeWindow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles remWindow.Click
        If windowListBox.SelectedIndex <> -1 Then
            widgetPropertiesGroup.Enabled = False
            GroupBox2.Enabled = False

            Dim index As Integer = findWidgetIndex(selectedWindow, windowListBox)
            ' DESTROY CHILD WIDGETS
            For Each i As Widget In selectedWindow.Controls
                destroyWidget(i) ' DESTROY WIDGET AND ASSOCIATED RESOURCES
            Next
            widgetListBox.Items.Clear()
            selectedWindow.Dispose()

            windowListBox.Items.RemoveAt(index)
            windowListBox.Refresh()

            If windowListBox.Items.Count = 0 Then
                selectedWindow = Nothing
            End If

            If windowListBox.Items.Count = 0 Then
                remWindow.Enabled = False
                windowListBox.Enabled = False
                exportLUA.Enabled = False
            End If
            GC.Collect()
        End If
    End Sub
    ' LOCATE WIDGETS LISTBOX INDEX
    Private Function findWidgetIndex(ByVal item As Widget, ByVal list As ListBox) As Integer
        Dim index As Integer = 0
        For Each w As WidgetItem In list.Items
            If w.mWidget.Equals(item) Then
                Return index
            End If
            index = index + 1
        Next
        Return -1
    End Function

    ' NEW WIDGET CREATION
    Private Sub newWidget(ByVal name As String, ByVal type As Integer)
        If atlasImage IsNot Nothing Then
            UID = UID + 1
            Dim widget As New Widget(name, type)
            selectedWindow.Controls.Add(widget)
            widgetListBox.Items.Add(New WidgetItem(widget))
            selectWidget(widget)
            remWidget.Enabled = True
        Else
            MsgBox("Unable to comply, no atlas loaded!")
        End If
    End Sub

    Private Sub newWindow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles addWindow.Click
        UID = UID + 1

        Dim window As New Widget("WINDOW", widgetType.WINDOW)

        Panel1.Controls.Add(window)

        windowListBox.Items.Add(New WidgetItem(window))
        selectWidget(window)

        widgetPropertiesGroup.Enabled = True
        GroupBox2.Enabled = True
        remWindow.Enabled = True
        windowListBox.Enabled = True
        exportLUA.Enabled = True
        saveButton.Enabled = True

    End Sub
    Private Sub newButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click
        newWidget("BUTTON", widgetType.BUTTON)
    End Sub
    Private Sub newCheckBox_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        newWidget("CHECKBOX", widgetType.CHECKBOX)
    End Sub
    Private Sub newRadioButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button5.Click
        newWidget("RADIOBUTTON", widgetType.RADIOBUTTON)
    End Sub
    Private Sub newTextView_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button6.Click
        newWidget("TEXTVIEW", widgetType.TEXTVIEW)
    End Sub
    Private Sub newPanel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button7.Click
        newWidget("PANEL", widgetType.PANEL)
    End Sub
    Private Sub newLabel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button8.Click
        newWidget("LABEL", widgetType.LABEL)
    End Sub
    Private Sub newLog_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button9.Click
        newWidget("LOG", widgetType.LOG)
    End Sub
    Private Sub newList_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button10.Click
        newWidget("LIST", widgetType.LIST)
    End Sub
    Private Sub newTextInput_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button11.Click
        newWidget("TEXTINPUT", widgetType.TEXTINPUT)
    End Sub
    Private Sub newHScrollBar_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        newWidget("HSCROLLBAR", widgetType.HSCROLLBAR)
    End Sub
    Private Sub newVScrollBar_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button12.Click
        newWidget("VSCROLLBAR", widgetType.VSCROLLBAR)
    End Sub
    Private Sub widgetNewImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button13.Click
        OpenFileDialog1.Title = "Select Image"
        OpenFileDialog1.FileName = ""
        OpenFileDialog1.InitialDirectory = currentDirectoryName
        OpenFileDialog1.Filter = "PNG Files|*.png"
        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then

            newWidget("IMAGE", widgetType.IMAGE)

            selectedWidget.mImage = Bitmap.FromFile(OpenFileDialog1.FileName)
            selectedWidget.mBitmap = New Bitmap(selectedWidget.mImage)
            selectedWidget.Width = selectedWidget.mBitmap.Width
            selectedWidget.Height = selectedWidget.mBitmap.Height

            Dim withoutFolder As String = OpenFileDialog1.FileName.Replace(currentDirectoryName & "\", "")
            selectedWidget.mImageName = (withoutFolder)
            widgetImage.Text = (withoutFolder)
        End If
    End Sub

    ' SET WIDGET PROPERTIES
    Private Sub widgetName_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetName.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            If selectedWidget.mType = widgetType.WINDOW Then
                ' REFRESH ITEM IN WINDOW LIST
                Dim index As Integer = findWidgetIndex(selectedWidget, windowListBox)
                If index <> -1 Then
                    selectedWidget.mName = widgetName.Text
                    windowListBox.RefreshItem(index)
                End If
            Else
                ' REFRESH ITEM IN WIDGET LIST
                Dim index As Integer = findWidgetIndex(selectedWidget, widgetListBox)
                If index <> -1 Then
                    selectedWidget.mName = widgetName.Text
                    widgetListBox.RefreshItem(index)
                End If
            End If
        End If
    End Sub
    Private Sub widgetLeft_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetLeft.ValueChanged
        selectedWidget.Left = CInt(widgetLeft.Value)
    End Sub
    Private Sub widgetTop_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetTop.ValueChanged
        selectedWidget.Top = CInt(widgetTop.Value)
    End Sub
    Private Sub widgetWidth_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetWidth.ValueChanged
        selectedWidget.Width = CInt(widgetWidth.Value)
    End Sub
    Private Sub widgetHeight_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetHeight.ValueChanged
        selectedWidget.Height = CInt(widgetHeight.Value)
    End Sub
    Private Sub widgetCaption_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetCaption.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mCaption = widgetCaption.Text
            selectedWidget.Invalidate()
        End If
    End Sub
    Private Sub widgetHasControls_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetHasControls.SelectedIndexChanged
        selectedWidget.mHasControls = widgetHasControls.SelectedIndex
    End Sub
    Private Sub widgetTextAlign_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetTextAlign.SelectedIndexChanged
        selectedWidget.mTextAlign = widgetTextAlign.SelectedIndex
        selectedWidget.Invalidate()
    End Sub
    Private Sub widgetAlpha_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetAlpha.Scroll
        selectedWidget.mColorARGB(0) = widgetAlpha.Value
        Dim a As Single = CSng(1 / widgetAlpha.Maximum * widgetAlpha.Value)
        widgetAlphaLabel.Text = a.ToString("F2")
        Panel1.Invalidate()
    End Sub
    Private Sub widgetTextAlpha_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetTextAlpha.Scroll
        selectedWidget.mTextColorARGB(0) = widgetTextAlpha.Value
        Dim a As Single = CSng(1 / widgetTextAlpha.Maximum * widgetTextAlpha.Value)
        widgetTextAlphaLabel.Text = a.ToString("F2")
    End Sub
    Private Sub widgetColor_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetColor.KeyPress
        Dim allowedChars As String = "0123456789abcdefABCDEF"
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            If widgetColor.Text <> "" Then
                selectedWidget.mColor = widgetColor.Text
                Try
                    selectedWidget.mColorARGB(1) = Convert.ToByte(widgetColor.Text.Substring(0, 2), 16)
                    selectedWidget.mColorARGB(2) = Convert.ToByte(widgetColor.Text.Substring(2, 2), 16)
                    selectedWidget.mColorARGB(3) = Convert.ToByte(widgetColor.Text.Substring(4, 2), 16)
                Catch ex As Exception
                    MsgBox("Invalid color!")
                    widgetColor.Text = ""
                    selectedWidget.mColorARGB(1) = 0
                    selectedWidget.mColorARGB(2) = 0
                    selectedWidget.mColorARGB(3) = 0
                    selectedWidget.mColor = ""
                    Panel4.BackColor = Color.FromKnownColor(KnownColor.Control)
                End Try
                Panel4.BackColor = Color.FromArgb(selectedWidget.mColorARGB(1), selectedWidget.mColorARGB(2), selectedWidget.mColorARGB(3))
            Else
                selectedWidget.mColor = ""
                Panel4.BackColor = Color.FromKnownColor(KnownColor.Control)
            End If

        ElseIf allowedChars.IndexOf(e.KeyChar) = -1 Then
            e.Handled = True
        End If
    End Sub
    Private Sub widgetTextColor_KeyPress1(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetTextColor.KeyPress
        Dim allowedChars As String = "0123456789abcdefABCDEF"
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            If widgetTextColor.Text <> "" Then
                selectedWidget.mTextColor = widgetTextColor.Text
                Try
                    selectedWidget.mTextColorARGB(1) = Convert.ToByte(widgetTextColor.Text.Substring(0, 2), 16)
                    selectedWidget.mTextColorARGB(2) = Convert.ToByte(widgetTextColor.Text.Substring(2, 2), 16)
                    selectedWidget.mTextColorARGB(3) = Convert.ToByte(widgetTextColor.Text.Substring(4, 2), 16)
                Catch ex As Exception
                    MsgBox("Invalid color!")
                    widgetTextColor.Text = ""
                    selectedWidget.mTextColorARGB(1) = 0
                    selectedWidget.mTextColorARGB(2) = 0
                    selectedWidget.mTextColorARGB(3) = 0
                    selectedWidget.mTextColor = ""
                    Panel3.BackColor = Color.FromKnownColor(KnownColor.Control)
                End Try
                Panel3.BackColor = Color.FromArgb(selectedWidget.mTextColorARGB(1), selectedWidget.mTextColorARGB(2), selectedWidget.mTextColorARGB(3))
            Else
                selectedWidget.mTextColor = ""
                Panel3.BackColor = Color.FromKnownColor(KnownColor.Control)
            End If

        ElseIf allowedChars.IndexOf(e.KeyChar) = -1 Then
            e.Handled = True
        End If
    End Sub
    Private Sub widgetTextColorPanel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Panel3.Click
        Dialog1.alpha = selectedWidget.mTextColorARGB(0)
        Dialog1.red = selectedWidget.mTextColorARGB(1)
        Dialog1.green = selectedWidget.mTextColorARGB(2)
        Dialog1.blue = selectedWidget.mTextColorARGB(3)

        Dialog1.ShowDialog()
        If Dialog1.DialogResult = System.Windows.Forms.DialogResult.OK Then
            widgetTextColor.Text = Dialog1.rgbString
            selectedWidget.mTextColor = Dialog1.rgbString
            selectedWidget.mTextColorARGB(0) = Dialog1.alpha
            selectedWidget.mTextColorARGB(1) = Dialog1.red
            selectedWidget.mTextColorARGB(2) = Dialog1.green
            selectedWidget.mTextColorARGB(3) = Dialog1.blue
            setWidgetColor(selectedWidget)
            Panel3.BackColor = Color.FromArgb(Dialog1.red, Dialog1.green, Dialog1.blue)
        End If
    End Sub
    Private Sub setWidgetColor(ByVal widget As Widget)
        Dim a As Integer = widget.mColorARGB(0)
        Dim r As Integer = widget.mColorARGB(1)
        Dim g As Integer = widget.mColorARGB(2)
        Dim b As Integer = widget.mColorARGB(3)
        widget.BackColor = Color.FromArgb(a, r, g, b)
    End Sub
    Private Sub widgetColorPanel_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Panel4.Click
        Dialog1.alpha = selectedWidget.mColorARGB(0)
        Dialog1.red = selectedWidget.mColorARGB(1)
        Dialog1.green = selectedWidget.mColorARGB(2)
        Dialog1.blue = selectedWidget.mColorARGB(3)

        Dialog1.ShowDialog()
        If Dialog1.DialogResult = System.Windows.Forms.DialogResult.OK Then
            widgetColor.Text = Dialog1.rgbString
            selectedWidget.mColor = Dialog1.rgbString
            selectedWidget.mColorARGB(0) = Dialog1.alpha
            selectedWidget.mColorARGB(1) = Dialog1.red
            selectedWidget.mColorARGB(2) = Dialog1.green
            selectedWidget.mColorARGB(3) = Dialog1.blue
            setWidgetColor(selectedWidget)
            Panel4.BackColor = Color.FromArgb(Dialog1.red, Dialog1.green, Dialog1.blue)
        End If
    End Sub
    Private Sub widgetBorderType_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetBorderType.SelectedIndexChanged
        selectedWidget.mBorderType = widgetBorderType.SelectedIndex
        If widgetBorderType.SelectedIndex = borderType.NONE And selectedWidget.mType = widgetType.WINDOW Then
            selectedWidget.Left = 0
            selectedWidget.Top = 0
            selectedWidget.Width = Panel1.Width
            selectedWidget.Height = Panel1.Height

        End If
        selectedWidget.Invalidate()
    End Sub
    Private Sub widgetOrientation_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetOrientation.SelectedIndexChanged
        selectedWidget.mOrientation = widgetOrientation.SelectedIndex
    End Sub
    Private Sub widgetEnabled_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetEnabled.SelectedIndexChanged
        selectedWidget.mEnabled = widgetEnabled.SelectedIndex
    End Sub
    Private Sub widgetHidden_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetHidden.SelectedIndexChanged
        selectedWidget.mHidden = widgetHidden.SelectedIndex
    End Sub
    Private Sub widgetTogglestate_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetTogglestate.SelectedIndexChanged
        selectedWidget.mToggleState = widgetTogglestate.SelectedIndex
        selectedWidget.Invalidate()
    End Sub
    Private Sub widgetToggleGroup_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetToggleGroup.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mToggleGroup = widgetToggleGroup.Text
        End If
    End Sub
    Private Sub widgetOnStateChanged_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnStateChanged.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnStateChanged = widgetOnStateChanged.Text
        End If
    End Sub
    Private Sub widgetInputType_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetInputType.SelectedIndexChanged
        selectedWidget.mInputType = widgetInputType.SelectedIndex
    End Sub
    Private Sub widgetRows_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetRows.ValueChanged
        selectedWidget.mRows = CInt(widgetRows.Value)
    End Sub
    Private Sub widgetVisibleLines_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetVisibleLines.ValueChanged
        selectedWidget.mVisibleLines = CInt(widgetVisibleLines.Value)
    End Sub
    Private Sub widgetList_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetList.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mList = widgetList.Text
        End If
    End Sub
    Private Sub widgetOnTextChange_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnTextChange.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnTextChanged = widgetOnTextChange.Text
        End If
    End Sub
    Private Sub widgetMin_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetMin.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mMin = widgetMin.Text
        End If
    End Sub
    Private Sub widgetMax_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetMax.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mMax = widgetMax.Text
        End If
    End Sub
    Private Sub widgetSteps_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetSteps.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mSteps = widgetSteps.Text
        End If
    End Sub
    Private Sub widgetValue_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetValue.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mValue = widgetValue.Text
        End If
    End Sub
    Private Sub widgetOnValueChanged_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnValueChanged.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnValueChanged = widgetOnValueChanged.Text
        End If
    End Sub
    Private Sub widgetOnPress_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnPress.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnPress = widgetOnPress.Text
        End If
    End Sub
    Private Sub widgetOnRelease_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnRelease.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnRelease = widgetOnRelease.Text
        End If
    End Sub
    Private Sub widgetOnDrag_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnDrag.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnDrag = widgetOnDrag.Text
        End If
    End Sub
    Private Sub widgetIndexChanged_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetIndexChanged.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnIndexChanged = widgetIndexChanged.Text
        End If
    End Sub
    Private Sub widgetItemRemoved_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetItemRemoved.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnItemRemoved = widgetItemRemoved.Text

        End If
    End Sub
    Private Sub widgetOnClose_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnClose.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnClose = widgetOnClose.Text
        End If
    End Sub
    Private Sub widgetOnOpen_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetOnOpen.KeyPress
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mOnOpen = widgetOnOpen.Text
        End If
    End Sub
    Private Sub widgetZIndex_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetZIndex.Scroll
        Dim z As Single = CSng(1 / 100 * widgetZIndex.Value)
        widgetZIndexLabel.Text = z.ToString("F2")
        selectedWidget.mZIndex = z
        selectedWindow.Invalidate()
    End Sub
    Private Sub widgetScaleX_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetScaleX.KeyPress
        Dim allowedChars As String = "0123456789."
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mScaleX = widgetScaleX.Text
        ElseIf allowedChars.IndexOf(e.KeyChar) = -1 Then
            e.Handled = True
        End If
    End Sub
    Private Sub widgetScaleY_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetScaleY.KeyPress
        Dim allowedChars As String = "0123456789."
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mScaleY = widgetScaleY.Text
        ElseIf allowedChars.IndexOf(e.KeyChar) = -1 Then
            e.Handled = True
        End If
    End Sub
    Private Sub widgetImageSelect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetImageSelect.Click
        OpenFileDialog1.Title = "Select Image"
        OpenFileDialog1.FileName = ""
        OpenFileDialog1.InitialDirectory = currentDirectoryName
        OpenFileDialog1.Filter = "PNG Files|*.png"
        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then

            Dim w As Widget = selectedWidget
            w.mBitmap.Dispose()
            w.mImage.Dispose()
            Try
                w.mImage = Bitmap.FromFile(OpenFileDialog1.FileName)
                w.mBitmap = New Bitmap(w.mImage)
                selectedWidget.Width = w.mBitmap.Width
                selectedWidget.Height = w.mBitmap.Height

                Dim withoutFolder As String = OpenFileDialog1.FileName.Replace(currentDirectoryName & "\", "")
                selectedWidget.mImageName = (withoutFolder)
                widgetImage.Text = (withoutFolder)
            Catch ex As Exception
                MsgBox("unable to load image " & OpenFileDialog1.FileName)
                w.mImage = noImageImage
                w.mBitmap = noImageBitmap
                w.mImageName = ""
            End Try
        End If

    End Sub
    Private Sub widgetRotation_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles widgetRotation.Scroll
        selectedWidget.mImageRotation = widgetRotation.Value
        Dim a As Single = CSng(1 / widgetRotation.Maximum * widgetRotation.Value)
        widgetRotationLabel.Text = widgetRotation.Value.ToString("F2")
    End Sub
    Private Sub widgetRotationLabel_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles widgetRotationLabel.KeyPress
        Dim allowedChars As String = "0123456789."
        If e.KeyChar = Chr(13) Then
            e.Handled = True
            selectedWidget.mImageRotation = CSng(widgetRotationLabel.Text)
            widgetRotation.Value = CInt(selectedWidget.mImageRotation)
        ElseIf allowedChars.IndexOf(e.KeyChar) = -1 Then
            e.Handled = True
        End If
    End Sub

    ' SELECT AND LOAD FONT
    Private Sub selectFont_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        If FontDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            fontLabel.Font = FontDialog1.Font
            fontLabel.Text = FontDialog1.Font.Name
            guiFont = FontDialog1.Font

            project.fontName = guiFont.Name
            project.fontSize = guiFont.Size
            project.fontStyle = guiFont.Style

            For Each window As WidgetItem In windowListBox.Items ' SET FONT FOR ALL WINDOWS AND CHILD WIDGETS
                window.mWidget.Font = guiFont
                For Each c As Control In window.mWidget.Controls
                    c.Font = guiFont
                Next
                window.mWidget.Invalidate()
            Next
        End If
    End Sub

    ' JSON ENCODING
    Public Class WidgetBuffer
        Public type As Integer
        Public x As Integer
        Public y As Integer
        Public width As Integer
        Public height As Integer
        Public name As String
        Public caption As String
        Public hasControls As Integer
        Public enabled As Integer
        Public hidden As Integer
        Public borderType As Integer
        Public alpha As String
        Public color As String
        Public colorARGB(3) As Integer
        Public textColor As String
        Public textColorARGB(3) As Integer
        Public textAlign As Integer
        Public orientation As Integer

        Public toggleState As Integer
        Public toggleGroup As String
        Public onStateChanged As String

        Public min As String
        Public max As String
        Public steps As String
        Public value As String
        Public onValueChanged As String

        Public list As String

        Public inputType As Integer
        Public onTextChanged As String

        Public rows As Integer
        Public visibleLines As Integer

        Public onPress As String
        Public onRelease As String
        Public onDrag As String
        Public onIndexChanged As String
        Public onItemRemoved As String
        Public onOpen As String
        Public onClose As String
        Public parentGroup As String
        Public numChildren As Integer
        Public zIndex As Single

        Public imageName As String
        Public scaleX As String
        Public scaleY As String
        Public imageRotation As Single

        Public mCanResizeX As Boolean
        Public mCanResizeY As Boolean
    End Class
    ' ENCODE A WIDGET TO A JSON STRING
    Private Function encodeWidget(ByVal w As Widget) As String
        Dim serializer As New JavaScriptSerializer()
        Dim widgetData As New WidgetBuffer With { _
                                   .type = w.mType, _
                                   .x = w.Left, _
                                   .y = w.Top, _
                                   .width = w.Width, _
                                   .height = w.Height, _
                                   .name = w.mName, _
                                   .caption = w.mCaption, _
                                   .hasControls = w.mHasControls, _
                                   .enabled = w.mEnabled, _
                                   .hidden = w.mHidden, _
                                   .borderType = w.mBorderType, _
                                   .alpha = w.mAlpha, _
                                   .color = w.mColor, _
                                   .colorARGB = w.mColorARGB, _
                                   .textColor = w.mTextColor, _
                                   .textColorARGB = w.mTextColorARGB, _
                                   .textAlign = w.mTextAlign, _
                                   .toggleState = w.mToggleState, _
                                   .toggleGroup = w.mToggleGroup, _
                                   .onStateChanged = w.mOnStateChanged, _
                                   .min = w.mMin, _
                                   .max = w.mMax, _
                                   .steps = w.mSteps, _
                                   .value = w.mValue, _
                                   .onValueChanged = w.mOnValueChanged, _
                                   .orientation = w.mOrientation, _
                                   .inputType = w.mInputType, _
                                   .onTextChanged = w.mOnTextChanged, _
                                   .rows = w.mRows, _
                                   .list = w.mList, _
                                   .visibleLines = w.mVisibleLines, _
                                   .onPress = w.mOnPress, _
                                   .onRelease = w.mOnRelease, _
                                   .onDrag = w.mOnDrag, _
                                   .onIndexChanged = w.mOnIndexChanged, _
                                   .onItemRemoved = w.mOnItemRemoved, _
                                   .onOpen = w.mOnOpen, _
                                   .onClose = w.mOnClose, _
                                   .parentGroup = w.mParentGroup, _
                                   .mCanResizeX = w.mCanResizeX, _
                                   .mCanResizeY = w.mCanResizeY, _
                                   .scaleX = w.mScaleX, _
                                   .scaleY = w.mScaleY, _
                                   .zIndex = w.mZIndex, _
                                   .imageName = w.mImageName, _
                                   .imageRotation = w.mImageRotation}

        If w.mType = widgetType.WINDOW Then
            widgetData.numChildren = widgetListBox.Items.Count
        End If

        Return serializer.Serialize(widgetData)
    End Function
    ' SAVE CURRENT PROJECT
    Private Sub saveButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles saveButton.Click
        If windowListBox.Items.Count > 0 Then
            SaveFileDialog1.Title = "Save GUI File"
            SaveFileDialog1.InitialDirectory = currentDirectoryName
            SaveFileDialog1.Filter = "GUI Files|*.gui"
            SaveFileDialog1.FileName = ""
            If SaveFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
                Try
                    currentDirectoryName = ExtractPathName(SaveFileDialog1.FileName)
                    Directory.SetCurrentDirectory(currentDirectoryName)

                    Dim sw As StreamWriter
                    Dim newLine As String = ""
                    Dim strb As New System.Text.StringBuilder()

                    Dim fileName As String = SaveFileDialog1.FileName
                    If File.Exists(fileName) Then
                        File.Delete(fileName) ' DELETE IT BEFORE RECREATING IT
                    End If
                    sw = File.CreateText(fileName)

                    Dim serializer As New JavaScriptSerializer() ' SERIALIZE AND WRITE PROJECT SETTINGS
                    newLine = serializer.Serialize(project) ' AHH ALL IN ONE HEHE
                    sw.WriteLine(newLine)

                    For Each window As WidgetItem In windowListBox.Items
                        newLine = encodeWidget(window.mWidget)
                        sw.WriteLine(newLine)
                        Dim parentGroup As String = window.mWidget.mName
                        For Each c As Control In window.mWidget.Controls
                            If TypeOf c Is Widget Then
                                ' WE HAVE TO TO RESORT TO THIS BECAUSE OF THE FAKE LABELS WE ARE USING TO ENABLE
                                ' WIDGET CAPTIONS TO APPEAR OUTSIDE THEIR BOUNDING BOXES *GRR*
                                Dim widget As Widget = CType(c, Widget)
                                widget.mParentGroup = parentGroup
                                newLine = encodeWidget(widget) ' SERIALIZE AND APPEND TO FILE
                                sw.WriteLine(newLine)
                            End If
                        Next
                    Next
                    sw.Flush()
                    sw.Close()
                Catch ex As Exception
                    MsgBox("Save Function: " & ex.Message)
                End Try
            End If
        Else
            MsgBox("SaveProject: there doesn't seem to be anything to save!")
        End If
    End Sub
    ' LOAD PROJECT
    Private Sub loadButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles loadButton.Click
        OpenFileDialog1.Title = "Select GUI File To Load"
        OpenFileDialog1.FileName = ""
        OpenFileDialog1.InitialDirectory = currentDirectoryName
        OpenFileDialog1.Filter = "GUI Files|*.gui"
        If OpenFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Try
                currentDirectoryName = ExtractPathName(OpenFileDialog1.FileName)
                Directory.SetCurrentDirectory(currentDirectoryName)

                destroyAllWidgets() ' PURGE ALL DATA
                Dim currentWindow As Widget
                Dim input As String = ""
                Dim jsonString As String
                Dim sr As StreamReader = File.OpenText(OpenFileDialog1.FileName)
                Dim serializer As New JavaScriptSerializer

                jsonString = sr.ReadLine() ' FIRST LINE OF THE FILE IS ALWAYS THE PROJECT SETTINGS
                Dim p As projectSettings = serializer.Deserialize(Of projectSettings)(jsonString)
                project.Width = p.Width
                project.Height = p.Height
                project.fontName = p.fontName
                project.fontSize = p.fontSize
                project.fontStyle = p.fontStyle


                Dim guiFont As New Font(project.fontName, project.fontSize, project.fontStyle)
                fontLabel.Font = guiFont
                fontLabel.Text = project.fontName
                FontDialog1.Font = guiFont

                Panel1.Font = guiFont
                Panel1.Width = project.Width
                Panel1.Height = project.Height

                Do While sr.Peek() <> -1 ' PROCESS REMAINDER OF FILE LINE BY LINE
                    jsonString = sr.ReadLine()

                    Dim b As WidgetBuffer = serializer.Deserialize(Of WidgetBuffer)(jsonString) 'decodeWidget(jsonString)
                    Dim w As New Widget(b.name, b.type)

                    w.mType = b.type
                    w.Left = b.x
                    w.Top = b.y
                    w.Width = b.width
                    w.Height = b.height
                    w.mName = b.name
                    w.mCaption = b.caption
                    w.mHasControls = b.hasControls
                    w.mEnabled = b.enabled
                    w.mHidden = b.hidden
                    w.mBorderType = b.borderType
                    w.mAlpha = b.alpha

                    w.mColor = b.color
                    w.mColorARGB = b.colorARGB

                    w.mTextColor = b.textColor
                    w.mTextColorARGB = b.textColorARGB

                    w.mTextAlign = b.textAlign

                    w.mToggleState = b.toggleState
                    w.mToggleGroup = b.toggleGroup
                    w.mOnStateChanged = b.onStateChanged

                    w.mMin = b.min
                    w.mMax = b.max
                    w.mSteps = b.steps
                    w.mValue = b.value
                    w.mOnValueChanged = b.onValueChanged
                    w.mOrientation = b.orientation

                    w.mInputType = b.inputType
                    w.mOnTextChanged = b.onTextChanged

                    w.mList = b.list
                    w.mRows = b.rows
                    w.mVisibleLines = b.visibleLines
                    w.mOnPress = b.onPress
                    w.mOnRelease = b.onRelease
                    w.mOnDrag = b.onDrag
                    w.mOnIndexChanged = b.onIndexChanged
                    w.mOnItemRemoved = b.onItemRemoved
                    w.mOnOpen = b.onOpen
                    w.mOnClose = b.onClose
                    w.mParentGroup = b.parentGroup
                    w.mZIndex = b.zIndex
                    w.mImageName = b.imageName
                    w.mImageRotation = b.imageRotation

                    If w.mType = widgetType.IMAGE And w.mImageName <> "" Then
                        Try
                            w.mImage = Bitmap.FromFile(w.mImageName)
                            w.mBitmap = New Bitmap(w.mImage)
                        Catch ex As Exception
                            MsgBox("unable to load image " & b.imageName)
                            w.mImage = noImageImage
                            w.mBitmap = noImageBitmap
                            w.mImageName = ""
                        End Try
                    End If

                    ' ADD TO LIST OF WINDOWS IF ITS A WINDOW WIDGET
                    If b.type = widgetType.WINDOW Then
                        currentWindow = w
                        Panel1.Controls.Add(w)
                        Panel1.Controls.SetChildIndex(w, 0)
                        w.Enabled = False
                        w.Visible = False
                        windowListBox.Items.Add(New WidgetItem(w))
                    Else
                        ' OTHERWISE ADD IT AS A CHILD OF THIS WIDGET (WHICH WILL BE A WINDOW WIDGET)
                        currentWindow.Controls.Add(w)
                    End If
                Loop
                sr.Close()

                ' POPULATE WIDGET LIST AND SELECT FIRST WINDOW
                Dim witem As WidgetItem = CType(windowListBox.Items(0), WidgetItem)
                Dim window As Widget = witem.mWidget
                If window.Controls.Count > 0 Then
                    For Each c As Control In window.Controls
                        If TypeOf c Is Widget Then
                            widgetListBox.Items.Add(New WidgetItem(CType(c, Widget)))
                        End If
                    Next
                End If
                'For Each w As Widget In window.Controls
                '    widgetListBox.Items.Add(New WidgetItem(w))
                ''Next
                'End If
                selectedWindow = window
                selectedWidget = window
                windowListBox.Enabled = True
                windowListBox.SelectedIndex = 0
                window.Invalidate() ' FORCE WINDOW TO REDRAW ITSSELF

                'GroupBox3.Enabled = True
                GroupBox1.Enabled = True
                GroupBox5.Enabled = True
                saveButton.Enabled = True
                exportLUA.Enabled = True
                exportJSON.Enabled = True

            Catch ex As Exception
                MsgBox("Load Function: " & ex.Message)
            End Try
        End If
    End Sub

    ' MODIFY EXPORT SETTINGS
    Private Sub exportSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles exportSettings.Click
        Dialog3.singleLineCheckBox.Checked = singleLineMode ' GET CURRENT SETTINGS
        Dialog3.padOutputCheckBox.Checked = padOutput
        Dialog3.padWidthTextBox.Text = padWidth.ToString
        Dialog3.tabWidthTextBox.Text = tabWidth.ToString

        If Dialog3.ShowDialog() = Windows.Forms.DialogResult.OK Then
            singleLineMode = Dialog3.singleLineCheckBox.Checked ' SAVE NEW SETTINGS
            padOutput = Dialog3.padOutputCheckBox.Checked
            padWidth = CInt(Dialog3.padWidthTextBox.Text)
            tabWidth = CInt(Dialog3.tabWidthTextBox.Text)
        End If

    End Sub

    ' EXPORT WIDGET PROPERTY
    Private Function indent(ByVal indentAmount As Integer, ByVal str As String) As String
        If singleLineMode Then
            Return str & "="
        Else
            Dim a As String = "".PadRight(indentAmount) & str
            If padOutput Then
                Return a.PadRight(padWidth) & " = "
            Else
                Return a & "="
            End If
        End If
    End Function
    Private Sub appendTypeProp(ByVal type As Integer)
        strBuilder.Append("".PadRight(tabWidth) & guiCall)
        Select Case type
            Case widgetType.WINDOW
                strBuilder.Append("newWindow({" & eol)
            Case widgetType.PANEL
                strBuilder.Append("newBorder({" & eol)
            Case widgetType.BUTTON
                strBuilder.Append("newButton({" & eol)
            Case widgetType.LABEL
                strBuilder.Append("newLabel({" & eol)
            Case widgetType.LIST
                strBuilder.Append("newList({" & eol)
            Case widgetType.LOG
                strBuilder.Append("newLog({" & eol)
            Case widgetType.CHECKBOX
                strBuilder.Append("newCheckBox({" & eol)
            Case widgetType.RADIOBUTTON
                strBuilder.Append("newRadioButton({" & eol)
            Case widgetType.TEXTVIEW
                strBuilder.Append("newText({" & eol)
            Case widgetType.TEXTINPUT
                strBuilder.Append("newTextInput({" & eol)
            Case widgetType.HSCROLLBAR
                strBuilder.Append("newScrollBar({" & eol)
            Case widgetType.VSCROLLBAR
                strBuilder.Append("newScrollBar({" & eol)
            Case widgetType.IMAGE
                strBuilder.Append("newImage({" & eol)
        End Select
    End Sub
    Private Sub appendIntegerProp(ByVal prop As Integer, ByVal var As Integer)
        If variables(var).export Then
            Dim name As String = variables(var).name
            Select Case variables(var).type
                Case varType.tSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & """" & prop.ToString & """," & eol)
                Case varType.tHEXSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & "0x" & prop.ToString & "," & eol)
                Case varType.tINTEGER
                    strBuilder.Append(indent(tabWidth * 2, name) & prop.ToString & "," & eol)
                Case varType.tFLOAT
                    strBuilder.Append(indent(tabWidth * 2, name) & CSng(prop).ToString("F2") & "," & eol)
                Case varType.tBOOL
                    strBuilder.Append(indent(tabWidth * 2, name) & CBool(prop).ToString & "," & eol)
            End Select
        End If
    End Sub
    Private Sub appendFunctionProp(ByVal prop As String, ByVal var As Integer) ' FUNCTIONS CAN ONLY BE EXPORTED AS FUNCTIONS
        If variables(var).export And prop <> "" Then
            Dim name As String = variables(var).name
            strBuilder.Append(indent(tabWidth * 2, name) & "function(e) " & prop & "end," & eol)
        End If
    End Sub
    Private Sub appendBoolProp(ByVal prop As Integer, ByVal var As Integer)
        If variables(var).export Then
            Dim name As String = variables(var).name
            Select Case variables(var).type
                Case varType.tSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & """" & prop.ToString.ToLower & """," & eol)
                Case varType.tINTEGER
                    strBuilder.Append(indent(tabWidth * 2, name) & CInt(prop).ToString & "," & eol)
                Case varType.tFLOAT
                    strBuilder.Append(indent(tabWidth * 2, name) & CSng(prop).ToString("F2") & "," & eol)
                Case varType.tBOOL
                    strBuilder.Append(indent(tabWidth * 2, name) & CBool(prop).ToString.ToLower & "," & eol)
            End Select
        End If
    End Sub
    Private Sub appendStringProp(ByVal prop As String, ByVal var As Integer)
        If variables(var).export And prop <> "" Then
            Dim name As String = variables(var).name
            Select Case variables(var).type
                Case varType.tSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & """" & prop & """," & eol)
                Case varType.tVARIABLE
                    strBuilder.Append(indent(tabWidth * 2, name) & prop & "," & eol)
            End Select
        End If
    End Sub
    Private Sub appendHexStringProp(ByVal prop As String, ByVal var As Integer) 'HEXSTRING, STRING, INTEGER
        If variables(var).export And prop <> "" Then
            Dim name As String = variables(var).name
            Select Case variables(var).type
                Case varType.tSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & """" & prop & """," & eol)
                Case varType.tHEXSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & "0x" & prop & "," & eol)
                Case varType.tINTEGER
                    strBuilder.Append(indent(tabWidth * 2, name) & prop & "," & eol)
            End Select
        End If
    End Sub
    Private Sub appendFloatProp(ByVal prop As Single, ByVal var As Integer) 'STRING, INTEGER, FLOAT, BOOL
        If variables(var).export Then
            Dim name As String = variables(var).name
            Select Case variables(var).type
                Case varType.tSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & """" & prop.ToString("F2") & """," & eol)
                    'Case varType.tINTEGER
                    '    strBuilder.Append(indent(tabWidth * 2, name) & CInt(prop).ToString & "," & eol)
                Case varType.tFLOAT
                    strBuilder.Append(indent(tabWidth * 2, name) & prop.ToString("F2") & "," & eol)
                    'Case varType.tBOOL
                    '    strBuilder.Append(indent(tabWidth * 2, name) & CBool(prop).ToString.ToLower & "," & eol)
            End Select
        End If
    End Sub
    Private Sub appendProp(ByVal prop As Boolean, ByVal var As Integer)
        If variables(var).export Then
            Dim name As String = variables(var).name
            Select Case variables(var).type
                Case varType.tSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & """" & prop.ToString & """," & eol)
                Case varType.tHEXSTRING
                    strBuilder.Append(indent(tabWidth * 2, name) & "0x" & prop.ToString & "," & eol)
                Case varType.tINTEGER
                    strBuilder.Append(indent(tabWidth * 2, name) & prop.ToString & "," & eol)
                Case varType.tFLOAT
                    strBuilder.Append(indent(tabWidth * 2, name) & CSng(prop).ToString("F2") & "," & eol)
                Case varType.tBOOL
                    strBuilder.Append(indent(tabWidth * 2, name) & prop.ToString.ToLower & "," & eol)
            End Select
        End If
    End Sub
    Private Sub appendVariableProp(ByVal prop As String, ByVal var As Integer)
        If variables(var).export And prop <> "" Then
            strBuilder.Append(indent(tabWidth * 2, Name) & prop & "," & eol)
        End If
    End Sub

    ' EXPORT TO LUA
    Private Sub exportLUA_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles exportLUA.Click
        SaveFileDialog1.Title = "Export LUA File"
        SaveFileDialog1.InitialDirectory = currentDirectoryName
        SaveFileDialog1.Filter = "LUA Files|*.lua"
        SaveFileDialog1.FileName = ""
        If SaveFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Try
                currentDirectoryName = ExtractPathName(SaveFileDialog1.FileName)
                Directory.SetCurrentDirectory(currentDirectoryName)

                Dim sw As StreamWriter
                Dim fileName As String = SaveFileDialog1.FileName
                If File.Exists(fileName) Then
                    File.Delete(fileName) ' DELETE IT BEFORE RECREATING IT
                End If
                sw = File.CreateText(fileName)

                strBuilder.Length = 0

                Dim eol2 As String = ""
                eol = vbCrLf
                If singleLineMode Then ' DISABLE PADDING AND NEWLINES
                    tabWidth = 0
                    padWidth = 0
                    eol = " "
                    eol2 = vbCrLf
                End If

                For Each window As WidgetItem In windowListBox.Items ' EXPORT WINDOW PROPERTIES
                    Dim widget As Widget = window.mWidget

                    strBuilder.Append("local function open_" & widget.mName & "()" & eol)

                    appendTypeProp(widget.mType)
                    appendIntegerProp(widget.Left, 0)
                    appendIntegerProp(widget.Top, 1)
                    appendIntegerProp(widget.Width, 2)
                    appendIntegerProp(widget.Height, 3)
                    appendStringProp(widget.mName, 4)
                    appendStringProp(widget.mCaption, 6)
                    appendStringProp(widgetTextAlign.Items(widget.mTextAlign).ToString, 7)
                    appendBoolProp(widget.mEnabled, 8)
                    appendBoolProp(widget.mHidden, 9)
                    appendHexStringProp(widget.mColor.ToLower, 10)
                    appendFloatProp(CSng((1.0 / 255) * CSng(widget.mColorARGB(0))), 11)
                    appendHexStringProp(widget.mTextColor.ToLower, 12)
                    appendFloatProp(CSng((1.0 / 255) * CSng(widget.mTextColorARGB(0))), 13)
                    appendStringProp(widgetBorderType.Items(widget.mBorderType).ToString, 14)
                    appendFloatProp(widget.mZIndex, 15)
                    appendFunctionProp(widget.mOnOpen, 16)
                    appendFunctionProp(widget.mOnClose, 17)
                    appendBoolProp(widget.mHasControls, 18)
                    strBuilder.Append("".PadRight(tabWidth) & "})" & eol & eol2)

                    Dim parentGroup As String = window.mWidget.mName

                    For Each c As Control In window.mWidget.Controls ' EXPORT WIDGETS
                        If TypeOf c Is Widget Then
                            Dim w As Widget = CType(c, Widget)

                            appendTypeProp(w.mType)
                            appendIntegerProp(w.Left, 0)
                            appendIntegerProp(w.Top, 1)
                            appendIntegerProp(w.Width, 2)
                            appendIntegerProp(w.Height, 3)
                            appendStringProp(w.mName, 4)
                            appendStringProp(parentGroup, 5)
                            appendStringProp(w.mCaption, 6)
                            appendStringProp(widgetTextAlign.Items(w.mTextAlign).ToString, 7)
                            appendBoolProp(w.mEnabled, 8)
                            appendBoolProp(w.mHidden, 9)
                            appendHexStringProp(w.mColor.ToLower, 10)
                            appendFloatProp(CSng((1.0 / 255) * CSng(w.mColorARGB(0))), 11)
                            appendHexStringProp(w.mTextColor.ToLower, 12)
                            appendFloatProp(CSng((1.0 / 255) * CSng(w.mTextColorARGB(0))), 13)
                            appendStringProp(widgetBorderType.Items(w.mBorderType).ToString, 14)
                            appendFloatProp(w.mZIndex, 15)

                            Select Case w.mType
                                '    Case widgetType.PANEL
                                Case widgetType.BUTTON
                                    appendFunctionProp(w.mOnPress, 34)
                                    appendFunctionProp(w.mOnRelease, 35)
                                    appendFunctionProp(w.mOnDrag, 36)
                                    '    Case widgetType.LABEL

                                Case widgetType.LIST
                                    appendVariableProp(w.mList, 30)
                                    appendIntegerProp(w.mVisibleLines, 31)
                                    appendFunctionProp(w.mOnIndexChanged, 32)
                                    appendFunctionProp(w.mOnItemRemoved, 33)

                                Case widgetType.LOG
                                    appendIntegerProp(w.mRows, 29)

                                Case widgetType.CHECKBOX
                                    appendBoolProp(w.mToggleState, 19)
                                    appendStringProp(w.mToggleGroup, 20)
                                    appendFunctionProp(w.mOnStateChanged, 21)

                                Case widgetType.RADIOBUTTON
                                    appendBoolProp(w.mToggleState, 19)
                                    appendStringProp(w.mToggleGroup, 20)
                                    appendFunctionProp(w.mOnStateChanged, 21)

                                Case widgetType.TEXTVIEW
                                    appendIntegerProp(w.mVisibleLines, 31)

                                Case widgetType.TEXTINPUT
                                    appendStringProp(widgetInputType.Items(w.mInputType).ToString, 22)
                                    appendFunctionProp(w.mOnTextChanged, 23)

                                Case widgetType.HSCROLLBAR
                                    appendIntegerProp(CInt(w.mMin), 24)
                                    appendIntegerProp(CInt(w.mMax), 25)
                                    appendIntegerProp(CInt(w.mSteps), 26)
                                    appendIntegerProp(CInt(w.mValue), 27)
                                    appendFunctionProp(w.mOnValueChanged, 28)

                                Case widgetType.VSCROLLBAR
                                    appendIntegerProp(CInt(w.mMin), 24)
                                    appendIntegerProp(CInt(w.mMax), 25)
                                    appendIntegerProp(CInt(w.mSteps), 26)
                                    appendIntegerProp(CInt(w.mValue), 27)
                                    appendFunctionProp(w.mOnValueChanged, 28)

                                Case widgetType.IMAGE
                                    appendStringProp(w.mImageName.Replace("\", "/"), 37)
                                    appendFloatProp(CSng(w.mScaleX), 38)
                                    appendFloatProp(CSng(w.mScaleY), 39)
                                    appendFloatProp(CSng(w.mImageRotation), 40)
                                    appendFunctionProp(w.mOnPress, 34)
                                    appendFunctionProp(w.mOnRelease, 35)
                                    appendFunctionProp(w.mOnDrag, 36)
                            End Select
                            strBuilder.Append("".PadRight(tabWidth) & "})" & eol & eol2)

                        End If
                    Next
                    strBuilder.Append("end" & vbCrLf) ' END OF WINDOW
                Next

                sw.WriteLine(strBuilder.ToString()) ' WRITE FILE


                ' END OF EXPORT
                sw.Flush()
                sw.Close()
            Catch ex As Exception
                MsgBox("LUA Export: " & ex.Message)
            End Try
        End If
    End Sub

    ' LOAD PROGRAM SETTINGS
    Public Sub loadSettings()
        Dim fileName As String = applicationDirectory & "\default.cfg"
        If File.Exists(applicationDirectory & "\current.cfg") Then
            fileName = applicationDirectory & "\current.cfg"
        End If
        Try
            Dim input As String = ""
            Dim jsonString As String
            Dim sr As StreamReader = File.OpenText(fileName)
            Dim serializer As New JavaScriptSerializer
            Do While sr.Peek() <> -1 ' PROCESS FILE LINE BY LINE
                jsonString = sr.ReadLine()
                variables.Add(serializer.Deserialize(Of variable)(jsonString))
            Loop
            sr.Close()
        Catch ex As Exception
            MsgBox("Load Settings: " & ex.Message)
        End Try
    End Sub




    'For Each v As variable In variables
    '    TextBox1.AppendText(v.name & "," & v.type.ToString & "," & v.export.ToString & vbCrLf)
    'Next

    Private Sub exportJSON_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles exportJSON.Click



        If windowListBox.Items.Count > 0 Then
            SaveFileDialog1.Title = "Export JSON File"
            SaveFileDialog1.InitialDirectory = currentDirectoryName
            SaveFileDialog1.Filter = "JSON Files|*.json"
            SaveFileDialog1.FileName = ""
            If SaveFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
                Try
                    currentDirectoryName = ExtractPathName(SaveFileDialog1.FileName)
                    Directory.SetCurrentDirectory(currentDirectoryName)

                    Dim sw As StreamWriter
                    Dim newLine As String = ""

                    Dim fileName As String = SaveFileDialog1.FileName
                    If File.Exists(fileName) Then
                        File.Delete(fileName) ' DELETE IT BEFORE RECREATING IT
                    End If
                    sw = File.CreateText(fileName)

                    Dim strb As New System.Text.StringBuilder()
                    strb.Append("[")

                    Dim serializer As New JavaScriptSerializer() ' SERIALIZE AND WRITE PROJECT SETTINGS
                    strb.Append(serializer.Serialize(project) & "," & vbCrLf)

                    For Each window As WidgetItem In windowListBox.Items

                        strb.Append(encodeWidget(window.mWidget) & "," & vbCrLf)


                        Dim parentGroup As String = window.mWidget.mName
                        For Each c As Control In window.mWidget.Controls
                            If TypeOf c Is Widget Then
                                ' WE HAVE TO TO RESORT TO THIS BECAUSE OF THE FAKE LABELS WE ARE USING TO ENABLE
                                ' WIDGET CAPTIONS TO APPEAR OUTSIDE THEIR BOUNDING BOXES *GRR*
                                Dim widget As Widget = CType(c, Widget)
                                widget.mParentGroup = parentGroup
                                strb.Append(encodeWidget(widget) & "," & vbCrLf)
                            End If
                        Next
                    Next
                    strb.Append("]")
                    sw.WriteLine(strb.ToString()) ' WRITE FILE
                    sw.Flush()
                    sw.Close()
                Catch ex As Exception
                    MsgBox("exportJSON() " & ex.Message)
                End Try
            End If
        Else
            MsgBox("exportJSON() there doesn't seem to be anything to export!")
        End If


    End Sub
End Class
